@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: scardcontrol.c 5724 2011-05-09 09:04:07Z rousseau $
+ * $Id: scardcontrol.c 5990 2011-10-02 12:10:07Z rousseau $
  */
 
 #include <stdio.h>
@@ -138,6 +138,13 @@ static void parse_properties(unsigned char *bRecvBuffer, int length)
 					putchar(p[i]);
 				printf(NORMAL "\n");
 				break;
+			case PCSCv2_PART10_PROPERTY_bPPDUSupport:
+				PRINT_GREEN_HEX2(" bPPDUSupport", value);
+				if (value & 1)
+					printf("  PPDU is supported over SCardControl using FEATURE_CCID_ESC_COMMAND\n");
+				if (value & 2)
+					printf("  PPDU is supported over SCardTransmit\n");
+				break;
 			default:
 				printf(" Unknown tag: 0x%02X (length = %d)\n", tag, len);
 		}
@@ -211,8 +218,9 @@ int main(int argc, char *argv[])
 	DWORD pin_properties_ioctl = 0;
 	DWORD mct_readerdirect_ioctl = 0;
 	DWORD properties_in_tlv_ioctl = 0;
+	DWORD ccid_esc_command = 0;
 	SCARD_IO_REQUEST pioRecvPci;
- 	SCARD_IO_REQUEST pioSendPci;
+	SCARD_IO_REQUEST pioSendPci;
 	PCSC_TLV_STRUCTURE *pcsc_tlv;
 #if defined(VERIFY_PIN) | defined(MODIFY_PIN)
 	int offset;
@@ -376,6 +384,10 @@ int main(int argc, char *argv[])
 				PRINT_GREEN("Reader supports", "FEATURE_GET_TLV_PROPERTIES");
 				properties_in_tlv_ioctl = ntohl(pcsc_tlv[i].value);
 				break;
+			case FEATURE_CCID_ESC_COMMAND:
+				PRINT_GREEN("Reader supports", "FEATURE_CCID_ESC_COMMAND");
+				ccid_esc_command = ntohl(pcsc_tlv[i].value);
+				break;
 			default:
 				printf("Can't parse tag: " RED "0x%02X" NORMAL, pcsc_tlv[i].tag);
 		}
@@ -386,8 +398,8 @@ int main(int argc, char *argv[])
 	{
 		int value;
 
-		rv = SCardControl(hCard, properties_in_tlv_ioctl, secoder_info,
-			sizeof(secoder_info), bRecvBuffer, sizeof(bRecvBuffer), &length);
+		rv = SCardControl(hCard, properties_in_tlv_ioctl, NULL, 0,
+			bRecvBuffer, sizeof(bRecvBuffer), &length);
 		PCSC_ERROR_CONT(rv, "SCardControl(GET_TLV_PROPERTIES)")
 
 		printf("GET_TLV_PROPERTIES (" GREEN "%ld" NORMAL "): " GREEN, length);
@@ -420,6 +432,29 @@ int main(int argc, char *argv[])
 		printf("\n");
 	}
 
+#ifdef GET_GEMPC_FIRMWARE
+	if (ccid_esc_command)
+	{
+		/* get GemPC firmware */
+		printf("Get GemPC Firmware\n");
+
+		/* this is specific to Gemalto readers */
+		bSendBuffer[0] = 0x02;
+		rv = SCardControl(hCard, ccid_esc_command, bSendBuffer,
+			1, bRecvBuffer, sizeof(bRecvBuffer), &length);
+
+		printf(" Firmware: " GREEN);
+		for (i=0; i<length; i++)
+			printf("%02X ", bRecvBuffer[i]);
+		printf(NORMAL "\n");
+
+		bRecvBuffer[length] = '\0';
+		printf(" Firmware: " GREEN "%s" NORMAL" (length " GREEN "%ld" NORMAL " bytes)\n", bRecvBuffer, length);
+
+		PCSC_ERROR_CONT(rv, "SCardControl")
+	}
+#endif
+
 	if (0 == verify_ioctl)
 	{
 		printf("Reader %s does not support PIN verification\n",
@@ -644,7 +679,7 @@ int main(int argc, char *argv[])
 	pin_modify -> bmFormatString = 0x82;
 	pin_modify -> bmPINBlockString = 0x04;
 	pin_modify -> bmPINLengthFormat = 0x00;
-	pin_modify -> bInsertionOffsetOld = 0x00; 	/* offset from APDU start */
+	pin_modify -> bInsertionOffsetOld = 0x00;	/* offset from APDU start */
 	pin_modify -> bInsertionOffsetNew = 0x04;	/* offset from APDU start */
 	pin_modify -> wPINMaxExtraDigit = 0x0408;	/* Min Max */
 	pin_modify -> bConfirmPIN = 0x03;	/* b0 set = confirmation requested */
