@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h 5740 2011-05-12 14:17:17Z rousseau $
+ * $Id: ccid.h 6024 2011-10-10 13:03:49Z rousseau $
  */
 
 typedef struct
@@ -53,7 +53,7 @@ typedef struct
 	 * PIN support of the reader (directly from Class Descriptor)
 	 */
 	char bPINSupport;
-	
+
 	/*
 	 * Display dimensions of the reader (directly from Class Descriptor)
 	 */
@@ -126,6 +126,16 @@ typedef struct
 	 * USB serial number of the device (if any)
 	 */
 	char *sIFD_serial_number;
+
+	/*
+	 * USB iManufacturer string
+	 */
+	char *sIFD_iManufacturer;
+
+	/*
+	 * USB bcdDevice
+	 */
+	int IFD_bcdDevice;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -182,7 +192,7 @@ typedef struct
 #define MYSMARTPAD	0x09BE0002
 #define CHERRYXX44	0x046a0010
 #define CL1356D		0x0B810200
-#define REINER_SCT	0x0C4B0300 
+#define REINER_SCT	0x0C4B0300
 #define SEG			0x08E68000
 #define BLUDRIVEII_CCID	0x1B0E1078
 #define DELLSCRK    0x413C2101
