@@ -20,7 +20,7 @@
  */
 
 /*
- * $Id: ccid_serial.c 5382 2010-11-05 22:49:01Z rousseau $
+ * $Id: ccid_serial.c 6021 2011-10-10 13:00:23Z rousseau $
  */
 
 #include <stdio.h>
@@ -43,6 +43,7 @@
 #include "utils.h"
 #include "commands.h"
 #include "parser.h"
+#include "strlcpycat.h"
 
 #define SYNC 0x03
 #define CTRL_ACK 0x06
@@ -489,11 +490,11 @@ status_t OpenSerial(unsigned int reader_index, int channel)
 
 	if (channel < 0)
 	{
-		DEBUG_CRITICAL2("wrong port number: %d", (int) channel);
+		DEBUG_CRITICAL2("wrong port number: %d", channel);
 		return STATUS_UNSUCCESSFUL;
 	}
 
-	(void)snprintf(dev_name, sizeof(dev_name), "/dev/pcsc/%d", (int) channel);
+	(void)snprintf(dev_name, sizeof(dev_name), "/dev/pcsc/%d", channel);
 
 	return OpenSerialByName(reader_index, dev_name);
 } /* OpenSerial */
@@ -659,7 +660,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	if (p)
 	{
 		/* copy the second part of the string */
-		strncpy(reader_name, p+1, sizeof(reader_name));
+		strlcpy(reader_name, p+1, sizeof(reader_name));
 
 		/* replace ':' by '\0' so that dev_name only contains the device name */
 		*p = '\0';
@@ -792,6 +793,8 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	}
 
 	serialDevice[reader_index].ccid.sIFD_serial_number = NULL;
+	serialDevice[reader_index].ccid.sIFD_iManufacturer = NULL;
+	serialDevice[reader_index].ccid.IFD_bcdDevice = 0;
 
 	return STATUS_SUCCESS;
 } /* OpenSerialByName */
