@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 5740 2011-05-12 14:17:17Z rousseau $
+ * $Id: ccid_usb.c 6025 2011-10-10 13:04:38Z rousseau $
  */
 
 #define __CCID_USB__
@@ -239,7 +239,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 		return STATUS_UNSUCCESSFUL; \
 	} \
 	else \
-		DEBUG_INFO2(key ": %s", list_get_at(values, 0));
+		DEBUG_INFO2(key ": %s", (char *)list_get_at(values, 0));
 
 	/* general driver info */
 	GET_KEY("ifdManufacturerString", values)
@@ -321,7 +321,9 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				int interface;
 				int num = 0;
 				const unsigned char *device_descriptor;
+#if defined(USE_COMPOSITE_AS_MULTISLOT) || defined(__APPLE__)
 				int readerID = (vendorID << 16) + productID;
+#endif
 
 #ifdef USE_COMPOSITE_AS_MULTISLOT
 				static int static_interface = 1;
@@ -567,6 +569,22 @@ again:
 						usbDevice[reader_index].ccid.sIFD_serial_number
 							= strdup((char *)serial);
 				}
+
+				usbDevice[reader_index].ccid.sIFD_iManufacturer = NULL;
+				if (desc.iManufacturer)
+				{
+					unsigned char iManufacturer[128];
+					int ret;
+
+					ret = libusb_get_string_descriptor_ascii(dev_handle,
+							desc.iManufacturer, iManufacturer,
+							sizeof(iManufacturer));
+					if (ret > 0)
+						usbDevice[reader_index].ccid.sIFD_iManufacturer
+							= strdup((char *)iManufacturer);
+				}
+
+				usbDevice[reader_index].ccid.IFD_bcdDevice = desc.bcdDevice;
 				goto end;
 			}
 		}
@@ -719,6 +737,9 @@ status_t CloseUSB(unsigned int reader_index)
 		if (usbDevice[reader_index].ccid.sIFD_serial_number)
 			free(usbDevice[reader_index].ccid.sIFD_serial_number);
 
+		if (usbDevice[reader_index].ccid.sIFD_iManufacturer)
+			free(usbDevice[reader_index].ccid.sIFD_iManufacturer);
+
 		/* reset so that bSeq starts at 0 again */
 		if (DriverOptions & DRIVER_OPTION_RESET_ON_CLOSE)
 			(void)libusb_reset_device(usbDevice[reader_index].dev_handle);
@@ -998,6 +1019,15 @@ int ControlUSB(int reader_index, int requesttype, int request, int value,
 		requesttype, request, value, usbDevice[reader_index].interface,
 		bytes, size, usbDevice[reader_index].ccid.readTimeout);
 
+	if (ret < 0)
+	{
+		DEBUG_CRITICAL5("control failed (%d/%d): %d %s",
+			usbDevice[reader_index].bus_number,
+			usbDevice[reader_index].device_address, ret, strerror(errno));
+
+		return STATUS_UNSUCCESSFUL;
+	}
+
 	if (requesttype & 0x80)
 		DEBUG_XXD("receive: ", bytes, ret);
 
