@@ -19,7 +19,7 @@
 */
 
 /*
- * $Id: commands.c 5578 2011-02-04 12:41:58Z rousseau $
+ * $Id: commands.c 5907 2011-08-24 09:07:20Z rousseau $
  */
 
 #include <string.h>
@@ -287,7 +287,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
-	unsigned char cmd[11+14+CMD_BUF_SIZE];
+	unsigned char cmd[11+14+TxLength];
 	unsigned int a, b;
 	PIN_VERIFY_STRUCTURE *pvs;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
@@ -303,14 +303,6 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 0;	/* bPINOperation: PIN Verification */
 
-	/* 19 is the size of the PCSCv2 PIN verify structure
-	 * The equivalent CCID structure is only 14-bytes long */
-	if (TxLength > 19+CMD_BUF_SIZE) /* command too large? */
-	{
-		DEBUG_INFO3("Command too long: %d > %d", TxLength, 19+CMD_BUF_SIZE);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	if (TxLength < 19+4 /* 4 = APDU size */)	/* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 19+4);
@@ -318,8 +310,8 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	}
 
 	/* On little endian machines we are all set. */
-	/* If on big endian machine and caller is using host byte order */ 
-	if ((pvs->ulDataLength + 19  == TxLength) && 
+	/* If on big endian machine and caller is using host byte order */
+	if ((pvs->ulDataLength + 19  == TxLength) &&
 		(bei2i((unsigned char*)(&pvs->ulDataLength)) == pvs->ulDataLength))
 	{
 		DEBUG_INFO("Reversing order from big to little endian");
@@ -329,8 +321,8 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		pvs->wLangId = BSWAP_16(pvs->wLangId);
 		pvs->ulDataLength = BSWAP_32(pvs->ulDataLength);
 	}
-	/* At this point we now have the above 3 variables in little endian */ 
-	
+	/* At this point we now have the above 3 variables in little endian */
+
 	if (dw2i(TxBuffer, 15) + 19 != TxLength) /* ulDataLength field coherency */
 	{
 		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 15) + 19, TxLength);
@@ -518,14 +510,14 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength)
 {
-	unsigned char cmd[11+19+CMD_BUF_SIZE];
+	unsigned char cmd[11+19+TxLength];
 	unsigned int a, b;
 	PIN_MODIFY_STRUCTURE *pms;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	int old_read_timeout;
 	RESPONSECODE ret;
 #ifdef BOGUS_PINPAD_FIRMWARE
-	int bNumberMessages = 0; /* for GemPC Pinpad */
+	int bNumberMessage = 0; /* for GemPC Pinpad */
 #endif
 
 	pms = (PIN_MODIFY_STRUCTURE *)TxBuffer;
@@ -537,14 +529,6 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	cmd[9] = 0;
 	cmd[10] = 1;	/* bPINOperation: PIN Modification */
 
-	/* 24 is the size of the PCSC PIN modify structure
-	 * The equivalent CCID structure is only 18 or 19-bytes long */
-	if (TxLength > 24+CMD_BUF_SIZE) /* command too large? */
-	{
-		DEBUG_INFO3("Command too long: %d > %d", TxLength, 24+CMD_BUF_SIZE);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	if (TxLength < 24+4 /* 4 = APDU size */) /* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 24+4);
@@ -552,8 +536,8 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	}
 
 	/* On little endian machines we are all set. */
-	/* If on big endian machine and caller is using host byte order */ 
-	if ((pms->ulDataLength + 24  == TxLength) && 
+	/* If on big endian machine and caller is using host byte order */
+	if ((pms->ulDataLength + 24  == TxLength) &&
 		(bei2i((unsigned char*)(&pms->ulDataLength)) == pms->ulDataLength))
 	{
 		DEBUG_INFO("Reversing order from big to little endian");
@@ -563,7 +547,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		pms->wLangId = BSWAP_16(pms->wLangId);
 		pms->ulDataLength = BSWAP_32(pms->ulDataLength);
 	}
-	/* At this point we now have the above 3 variables in little endian */ 
+	/* At this point we now have the above 3 variables in little endian */
 
 
 	if (dw2i(TxBuffer, 20) + 24 != TxLength) /* ulDataLength field coherency */
@@ -598,17 +582,17 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if ((SPR532 == ccid_descriptor->readerID)
 		|| (CHERRYST2000 == ccid_descriptor->readerID))
 	{
-		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+		TxBuffer[11] = 0x03; /* set bNumberMessage to 3 so that
 								all bMsgIndex123 are filled */
 		TxBuffer[14] = TxBuffer[15] = TxBuffer[16] = 0;	/* bMsgIndex123 */
 	}
 
 	/* the bug is a bit different than for the Cherry ST 2000C
-	 * with bNumberMessages < 3 the command seems to be accepted
+	 * with bNumberMessage < 3 the command seems to be accepted
 	 * and the card sends 6B 80 */
 	if (CHERRYXX44 == ccid_descriptor->readerID)
 	{
-		TxBuffer[11] = 0x03; /* set bNumberMessages to 3 so that
+		TxBuffer[11] = 0x03; /* set bNumberMessage to 3 so that
 								all bMsgIndex123 are filled */
 	}
 
@@ -625,12 +609,14 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 			TxBuffer[10] = 0x02;	/* validation key pressed */
 		}
 
-		/* the reader does not support any other value than 3 for the number
-		 * of messages */
-		bNumberMessages = TxBuffer[11];
+		/* The reader requests a value for bMsgIndex2 and bMsgIndex3
+		 * even if they should not be present. So we fake
+		 * bNumberMessage=3.  The real number of messages will be
+		 * corrected later in the code */
+		bNumberMessage = TxBuffer[11];
 		if (0x03 != TxBuffer[11])
 		{
-			DEBUG_INFO2("Correct bNumberMessages for GemPC Pinpad (was %d)",
+			DEBUG_INFO2("Correct bNumberMessage for GemPC Pinpad (was %d)",
 				TxBuffer[11]);
 			TxBuffer[11] = 0x03; /* 3 messages */
 		}
@@ -698,18 +684,18 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		/* copy to the CCID block 'verbatim' */
 		cmd[a] = TxBuffer[b];
 		a++;
- 	}
+	}
 
 #ifdef BOGUS_PINPAD_FIRMWARE
 	if ((SPR532 == ccid_descriptor->readerID)
 		|| (CHERRYST2000 == ccid_descriptor->readerID))
 	{
-		cmd[21] = 0x00; /* set bNumberMessages to 0 */
+		cmd[21] = 0x00; /* set bNumberMessage to 0 */
 	}
 
 	if ((GEMPCPINPAD == ccid_descriptor->readerID)
 		|| (VEGAALPHA == ccid_descriptor->readerID))
-		cmd[21] = bNumberMessages;	/* restore the real value */
+		cmd[21] = bNumberMessage;	/* restore the real value */
 #endif
 
 	/* We know the size of the CCID message now */
@@ -720,11 +706,11 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
 	{
- 		ret = IFD_COMMUNICATION_ERROR;
+		ret = IFD_COMMUNICATION_ERROR;
 		goto end;
 	}
 
- 	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
+	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
 	/* T=1 Protocol Management for a TPDU reader */
 	if ((SCARD_PROTOCOL_T1 == ccid_descriptor->cardProtocol)
@@ -829,6 +815,7 @@ again:
 
 	if (length_out < STATUS_OFFSET+1)
 	{
+		free(cmd_out);
 		DEBUG_CRITICAL2("Not enough data received: %d bytes", length_out);
 		return_value = IFD_COMMUNICATION_ERROR;
 		goto end;
@@ -1117,7 +1104,7 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 				if (protocol == T_1)
 					return_value = CmdXfrBlockTPDU_T1(reader_index, tx_length,
 						tx_buffer, rx_length, rx_buffer);
- 				else
+				else
 					return_value = IFD_PROTOCOL_NOT_SUPPORTED;
 			break;
 
@@ -1137,7 +1124,7 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	const unsigned char tx_buffer[], unsigned short rx_length, unsigned char bBWI)
 {
-	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
+	unsigned char cmd[10+tx_length];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	status_t ret;
 
@@ -1192,13 +1179,6 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	cmd[8] = rx_length & 0xFF;	/* Expected length, in character mode only */
 	cmd[9] = (rx_length >> 8) & 0xFF;
 
-	/* check that the command is not too large */
-	if (tx_length > CMD_BUF_SIZE)
-	{
-		DEBUG_CRITICAL2("TX Length too big: %d", tx_length);
-		return IFD_NOT_SUPPORTED;
-	}
-
 	memcpy(cmd+10, tx_buffer, tx_length);
 
 	ret = WritePort(reader_index, 10+tx_length, cmd);
@@ -1438,7 +1418,7 @@ static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	unsigned char chain_parameter;
 	unsigned int local_tx_length, sent_length;
-	unsigned int local_rx_length, received_length;
+	unsigned int local_rx_length = 0, received_length;
 	int buffer_overflow = 0;
 
 	if (PROTOCOL_ICCD_B == ccid_descriptor->bInterfaceProtocol)
@@ -1787,8 +1767,6 @@ static RESPONSECODE T0ProcSW1(unsigned int reader_index,
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
 	UCHAR tmp_buf[512];
-	unsigned char *rcv_buf_tmp = rcv_buf;
-	const unsigned int rcv_len_tmp = *rcv_len;
 	unsigned char sw1, sw2;
 
 	/* store the SW1 */
@@ -1817,12 +1795,6 @@ static RESPONSECODE T0ProcSW1(unsigned int reader_index,
 	in_len--;
 	(*rcv_len)++;
 
-	if (return_value != IFD_SUCCESS)
-	{
-		rcv_buf_tmp[0] = rcv_buf_tmp[1] = 0;
-		*rcv_len = rcv_len_tmp;
-	}
-
 	DEBUG_COMM3("Exit: SW=%02X %02X", sw1, sw2);
 
 	return return_value;
@@ -2082,7 +2054,7 @@ static RESPONSECODE CmdXfrBlockTPDU_T1(unsigned int reader_index,
 RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	unsigned int length, unsigned char buffer[])
 {
-	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
+	unsigned char cmd[10+length];	/* CCID + APDU buffer */
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	DEBUG_COMM2("length: %d bytes", length);
@@ -2094,10 +2066,6 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 	cmd[7] = protocol;	/* bProtocolNum */
 	cmd[8] = cmd[9] = 0; /* RFU */
 
-	/* check that the command is not too large */
-	if (length > CMD_BUF_SIZE)
-		return IFD_NOT_SUPPORTED;
-
 	memcpy(cmd+10, buffer, length);
 
 	if (WritePort(reader_index, 10+length, cmd) != STATUS_SUCCESS)
